(* This module is part of the bachelor thesis "Lambda-Interpreter", *)
(* written by Michael Brunner and supervised by Christian Sternagel, *)
(* University of Innsbruck, Institute of Computer Science *)
(* Date: 01.02.2009 *)
(* Copyright 2009 Michael Brunner, you may use this code or parts *)
(* of it as long as you keep this copyright notice in tact *)

type var = Strng.t * int
type cons = True | False
type t =
    Var of var
  | Cons of cons
  | Abs of var * t
  | App of t * t
  | Let of var * t * t
  | If of t * t * t
type dir = Left | Middle | Right
type pos = dir list

val var : Strng.t -> t
val abs : Strng.t list -> t -> t
val app : t list -> t
val leti : Strng.t -> t -> t -> t
val ite : t -> t -> t -> t
val cons : cons -> t

val vars : t -> var St.t
val fvars : t -> var St.t
val bvars : t -> var St.t

val substitute : var -> t -> t -> t

val beta : t -> t
val is_redex : t -> bool
val redexpos : t -> pos list

val is_whnf : t -> bool
val is_nf : t -> bool
val is_closed : t -> bool

val pos : t -> t -> pos list
val get : t -> pos -> t

val replace : t -> t -> pos -> t
val replace_all : t -> (pos * t) list -> t
val reduce_vars : t -> t

module Strategies :
  sig
    val call_by_value : t -> (pos * t) list
    val call_by_name : t -> (pos * t) list
    val parallel_inner : t -> (pos * t) list
    val parallel_outer : t -> (pos * t) list
  end
val do_step : (t -> (pos * t) list) -> t -> t
val calc_steps : (t -> (pos * t) list) -> t -> (t * pos list) list
val calc_nf : (t -> (pos * t) list) -> t -> t

val parenthesis : 'a Parser.t -> 'a Parser.t
val varname : Strng.t -> char list Parser.result
val ltp : t Parser.t

val var2strng : char list * int -> char list
val to_strng' : t -> Strng.t
val to_strng : t -> Strng.t
val to_strng_mark' : pos list -> t -> Strng.t
val to_strng_mark : pos list -> t -> Strng.t
val toplevel_printer : Format.formatter -> t -> unit
