(* This module is part of the bachelor thesis "Lambda-Interpreter", *)
(* written by Michael Brunner and supervised by Christian Sternagel, *)
(* University of Innsbruck, Institute of Computer Science *)
(* Date: 01.02.2009 *)
(* Copyright 2009 Michael Brunner, you may use this code or parts *)
(* of it as long as you keep this copyright notice in tact *)

(** minimalistic monad-type, primary used for quick definitions *)
module type MONAD_MINIMAL = sig
	type 'a t
	val bind: 'a t -> ('a -> 'b t) -> 'b t
	val unit: 'a -> 'a t
end;;

(** useful monad-type *)
module type MONAD = sig 
	type 'a t
	val (>>=): 'a t -> ('a -> 'b t) -> 'b t
	val (>>): 'a t -> 'b t -> 'b t
	val return: 'a -> 'a t
	val run: 'a t -> 'a
end;;

(** module-type to represent generic states used by state-monad *)
module type TYPEWRAPPER = sig 
	type t;;
  val init: t;;
end;;


(** identity-monad *)
module Identity: sig
	include MONAD;;
(*	type 'a t;;
  val (>>=): 'a t -> ('a -> 'b t) -> 'b t;;
  val (>>): 'a t -> 'b t -> 'b t;;
  val return: 'a -> 'a t;;
  val run: 'a t -> 'a;;*)
  
end;;


(** option-monad *)
module Option: sig
  type 'a t;;
  val (>>=): 'a t -> ('a -> 'b t) -> 'b t;;
  val (>>): 'a t -> 'b t -> 'b t;;
  val return: 'a -> 'a t;;
  val run: 'a t -> 'a;;
  val fail: 'a t;;
  val on_fail: 'a t -> 'a t -> 'a t;;
end;;


(** exception-monad *)
module Exception: sig 
	include MONAD;;
(*	type 'a t;;
  val (>>=): 'a t -> ('a -> 'b t) -> 'b t;;
  val (>>): 'a t -> 'b t -> 'b t;;
  val return: 'a -> 'a t;;*)
  val raise: string -> 'a t;;
  val failed: 'a t -> bool;;
  val trywith: 'a t -> (string -> 'a t) -> 'a t;;
(*  val run: 'a t -> 'a;;*)
end;;


(** state-monad *)
module State (S: TYPEWRAPPER): sig 
	include MONAD;;
(*	type 'a t;;
  val (>>=): 'a t -> ('a -> 'b t) -> 'b t;;
  val (>>): 'a t -> 'b t -> 'b t;;
  val return: 'a -> 'a t;;*)
  val read_state: S.t t;;
  val write_state: S.t -> unit t;;
(*  val run: 'a t -> 'a;;*)
  val run_with: 'a t -> S.t -> 'a;;
end;;


(** io-monad *)
module IO: sig 
	include MONAD;;
(*	type 'a t;;
  val (>>=): 'a t -> ('a -> 'b t) -> 'b t;;
  val (>>): 'a t -> 'b t -> 'b t;;
  val return: 'a -> 'a t;;*)
  val write: string -> unit t;;
  val write_line: string -> unit t;;
  val read_line: string t;;
  val file_exists: string -> bool t;;
  val read_file: string -> string Lst.t t;;
  val write_file: string -> char Lst.t -> bool t;;
(*  val run: 'a t -> 'a;;*)
end;;
